/***************************************************************************
 *
 * Copyright (c) 1997-2011 CRI Middlware Co., LTD.
 *
 * Appli.   : ADX Decoder
 * Module   : Library
 * File     : adxdcd.c
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "adxdcd.h"

#define	CHAR_BIT	(8)

/* 萔 */
#define	ADX_FIXED_POINT_BIT		(12)		// Œ菬_lւ̃Vtgp萔

#define	ADX_CTRL_CODE_INFO		(0x8000)	// R[hp萔
#define	ADX_STREAM_HEADER_SIZE	(2)
#define	ADX_BLK_SIZE			(18)		// ubNTCY
#define	ADX_DEF_BPS				(4)
#define	ADX_DEF_NSMPL_BLK		((ADX_BLK_SIZE - ADX_STREAM_HEADER_SIZE) * CHAR_BIT / ADX_DEF_BPS)
#define	ADX_FIXED_POINT_OFFSET	(0x1000)	// Œ菬_lւ̕ϊp萔
#define	ADX_MAX_HEADER_SIZE		(56)
#define	ADX_GAIN_MASK			(0x1fff)

#define	ADX_SHRT_MIN			(-32768)
#define	ADX_SHRT_MAX			(32767)

/* }N */
#if 0 /*Little Endien = 1 : Big Endien = 0*/
  #define ADX_ConvBeShort(s)	(Sint16)((((s) >> 8) & 0x00ff) | (((s) << 8) & 0xff00))
#else
  #define ADX_ConvBeShort(s)	(Sint16)(s)
#endif

#define ADX_LoadBeShort(p)	((*((Uint8*)p)<< 8)&0xff00|(*((Uint8*)p+1))&0x00ff)
#define ADX_LoadBeLong(p)	((*((Uint8*)p)<<24)&0xff000000|(*((Uint8*)p+1)<<16)&0x00ff0000|(*((Uint8*)p+2)<<8)&0x0000ff00|(*((Uint8*)p+3))&0x000000ff)

#define ADX_ClipShort(d)	(dt < ADX_SHRT_MIN ? ADX_SHRT_MIN : ((dt > ADX_SHRT_MAX) ? ADX_SHRT_MAX : dt))  // OaZ

// 4 bit ʎqf[^̕ϊe[u
static const Sint32 g_adx_qtbl[16] = {
	0x00000000, 0x00000001, 0x00000002, 0x00000003,
	0x00000004, 0x00000005, 0x00000006, 0x00000007,
	0xfffffff8, 0xfffffff9, 0xfffffffa, 0xfffffffb,
	0xfffffffc, 0xfffffffd, 0xfffffffe, 0xffffffff
};

// ADXwb_̉
Sint8 *ADX_AnalyzeHeaderInfo(Sint8 *ibuf, Sint32 ibuflen, AdxHeaderInfo *inf)
{
	Uint16	type;
	Sint32	lpflg;
	Sint8	*ibufdt = (Sint8 *)ibuf;

	if (ibuflen < ADX_MAX_HEADER_SIZE)
		return NULL;

	// wb_̃`FbN
	type = ibufdt[0];	// type𐳂擾ł
	type = type <<  8;
	type = type | ibufdt[1];
	if (type != ADX_CTRL_CODE_INFO)
		inf->code			= 0x03;

	inf->dlen = ibufdt[2];	// type𐳂擾ł
	inf->dlen <<= 8;
	inf->dlen |= ibufdt[3];
	inf->dlen += 4;
	
	inf->code			= ibufdt[4];
	inf->blksize		= ibufdt[5];
	inf->bps			= ibufdt[6];
	inf->nch			= ibufdt[7];

	inf->sfreq			= ADX_LoadBeLong(&ibufdt[8]);
	inf->total_nsmpl	= ADX_LoadBeLong(&ibufdt[12]);

	// fBCl
	inf->dly_l[0]		= 0;
	inf->dly_l[1]		= 0;
	inf->dly_r[0]		= 0;
	inf->dly_r[1]		= 0;

	// fR[hubN
	inf->nsmpl_blk		= ((Sint32)inf->blksize - ADX_STREAM_HEADER_SIZE) * 8 / inf->bps;
	
	// WvZ
	if (inf->sfreq == 6000) {
		inf->k[0] = 3740;
		inf->k[1] = -853;
	} else if (inf->sfreq == 7000) {
		inf->k[0] = 4156;
		inf->k[1] = -1054;
	} else if (inf->sfreq == 8000) {
		inf->k[0] = 4507;
		inf->k[1] = -1240;
	} else if (inf->sfreq == 8192) {
		inf->k[0] = 4568;
		inf->k[1] = -1274;
	} else if (inf->sfreq == 8820) {
		inf->k[0] = 4755;
		inf->k[1] = -1380;
	} else if (inf->sfreq == 11025) {
		inf->k[0] = 5287;
		inf->k[1] = -1706;
	} else if (inf->sfreq == 12000) {
		inf->k[0] = 5475;
		inf->k[1] = -1829;
	} else if (inf->sfreq == 16000) {
		inf->k[0] = 6048;
		inf->k[1] = -2232;
	} else if (inf->sfreq == 22050) {
		inf->k[0] = 6569;
		inf->k[1] = -2634;
	} else if (inf->sfreq == 24000) {
		inf->k[0] = 6687;
		inf->k[1] = -2729;
	}

	return ibuf + inf->dlen;
}

// PubÑfR[h
Sint8 *ADX_DecodeBlock(Sint8 *ptr, Sint16 *obuf, Sint16 *dly, Sint16 *k)
{
	Sint32 	dt, d0, d1, sno, qsig1, qsig2;
	Sint16	gain, code;
	Sint8	*ibufdt;

    ibufdt = (Sint8*)ptr;

	// ÕfBCl̐ݒ
	d0 = dly[0];				
	d1 = dly[1];

	code = ADX_ConvBeShort(*(Sint16 *)ibufdt);
	if (code & ADX_CTRL_CODE_INFO) {				// R[hI
		return NULL;
	}

	gain = (Sint16)((code & ADX_GAIN_MASK) + 1);	// QC̐ݒ
	ibufdt += ADX_STREAM_HEADER_SIZE;

	for(sno = 0; sno < ADX_DEF_NSMPL_BLK; sno += 2) {
		qsig1 = qsig2 = *ibufdt;
		qsig1 = qsig1 >> 4;
		ibufdt++;

		dt = ((k[0] * d0 + k[1] * d1) >> ADX_FIXED_POINT_BIT) + qsig1 * gain;
		dt = ADX_ClipShort(dt);		// OaZ
		d1 = d0;
		d0 = dt;
		*obuf++ = (Sint16)d0;

		qsig2 = g_adx_qtbl[qsig2 & 0x0f];
		dt = ((k[0] * d0 + k[1] * d1) >> ADX_FIXED_POINT_BIT) + qsig2 * gain;
		dt = ADX_ClipShort(dt);		// OaZ
		d1 = d0;
		d0 = dt;
		*obuf++ = (Sint16)d0;
	}

	// 񗘗p
	dly[0] = (Sint16)d0;
	dly[1] = (Sint16)d1;

	return ibufdt;
}

/* end of file */
